C**************************************************************************
C************************ MCSKY version 2.3 *******************************
C**************************************************************************
C**                                                                      **
C**  This program calculates the skyshine dose from an isotropic, mono-  **
C**  energetic, point gamma ray source.  The source can be shielded or   **
C**  unshielded and the shield can be either a silo or a convex polygon. **
C**  The program uses a Monte Carlo approach to track the gamma ray      **
C**  through the shield and the approximate line-beam response function  **
C**  to calculate the transport through the atmosphere.                  **
C**                                                                      **
C**  The following subroutines and functions subprograms are used:       **
C**    MENU   - Generates a menu screen from which program options can   **
C**             be chosen.                                               **
C**    INP    - Prompts the user for the appropriate input varaibles     **
C**             necessary for program execution.                         **
C**    INFILE - Used to input the data from a file.                      **
C**    PBDSCR - Prints the problem data to the screen for review by the  **
C**             user.                                                    **
C**    INTIAL - Resets tally counters to zero after each batch run.      **
C**    PBDOUT - Outputs the problem data to a file in the correct format **
C**             necessary for INFILE.                                    **
C**    MCARLO - Controls the execution of the Monte Carlo tracking and   **
C**             calculation of the Line Beam Response Function.          **
C**    BIN    - Bins the gamma rays escaping after a scatter (Note:      **
C**             Not including pair production).                          **
C**    BINPP  - Bins gamma rays escaping directly from a pair production **
C**             interaction.                                             **
C**    BINMN  - Bins gamma rays escaping without interacting within the  **
C**             shield.                                                  **
C**    SOURCE - Calculation of source gamma ray direction and if a       **
C**             polygon shield is used determines if gamma ray is        **
C**             tracked or rejected.                                     **
C**    SHIELD - Performs the Monte Carlo game of tracking the gamma ray  **
C**             through the shield.                                      **
C**    CMPT   - Calculates the scattered Compton wavelength of a gamma   **
C**             ray undergoing Compton scattering using the              **
C**             Klein-Nishina cross section.                             **
C**    HOWFAR - Determines if the gamma ray has exited the shield and if **
C**             a laminate shield which shield the particle is in.       **
C**    SHIFT  - Calculates the azimuthal angle and the cosine of the     **
C**             polar angle after a scatter.                             **
C**    DIRCOS - Calculates the direction cosines of the gamma ray.       **
C**    PATHL  - Calculates a path length for the gamma ray when it       **
C**             enters the shield and after each interaction.            **
C**    COM    - Calculates the compton scattering cross section.         **
C**    PPAIR  - Calculates the pair production cross section.            **
C**    TOT    - Calculates the total cross section.                      **
C**    FLTRN  - Generates a double precision random number.              **
C**    GETWP  - Generates isotropic direction using the azimuthal angle  **
C**             and the cosine of the polar angle.                       **
C**    XLBRF  - Outputs the problem data and calculated responses.       **
C**    XBEAM  - Calculates the line beam response function for a given   **
C**             energy, angle and air density using 3 parameter approx.  **
C**    XBEAM4 - Calculates the line beam response function for a given   **
C**             energy, angle and air density using 4 parameter approx.  **
C**    BLOCK DATA - Contains data necessary to execute the problem.      **
C**                                                                      **
C**  Data Needs:                                                         **
C**     The files LOWGAM.DAT and HIGHGAM.DAT containing parameters for   **
C**     the approximation of the line-beam response function must be     **
C**     available to the program.                                        **
C**     The MCSKY code also has the ability to use a four parameter      **
C**     line-beam response function at low energies (<15 MeV).  The      **
C**     file LOWGAM4.DAT must be available to use this option.  The      **
C**     subroutine XBEAM4 is invoke by changing the value of the flag    **
C**     IPARM in the MCARLO subroutine from 3 to 4.                      **
C**                                                                      **
C**  Written by M. Stedry, R. Faw and K. Shultis, Kansas State Univ.     **
C**  Manhattan, KS 66502.                                                **
C**  Revised:  July 1994 (version 2.1)                                   **
C**            January 1995 (version 2.2)                                **
C**            October 1997 (version 2.3)                                **
C**************************************************************************
C**************************************************************************

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      CHARACTER*32 FILEIN, FILEOT, EDGEIN, FD
      CHARACTER*1 ERFLG

      COMMON /SENG/ ENERGY
      COMMON /NTARG/ NEDGE,NOUTP
      COMMON /TARGET/ XEDGE(101),YEDGE(101)
      COMMON /BINS/ NBE,NBA,NBATCH
      COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
      COMMON /DISTMA/ MAT(2)
      COMMON /RESPNC/ RESPEP,RESPPP,RESPMN
      COMMON /PHIX/ XPHI(20)
      COMMON /BINEP/ IEPHI(20,50,20)
      COMMON /PPBIN/ IPPPHI(20,20)
      COMMON /MNBIN/ ISPHI(20,20)
      COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)    !<<<<<<<<< Revised Oct 97, REF
      COMMON /DENSTY/ DENS(4)
      COMMON /LIMIT/ ELIM(51),PHILIM(21)
      COMMON /BATCH/ SUM1(20),SUM2(20)
      COMMON /PARKD/ SEED
      FD = '(1X,A,$)'

C--- Set freq. at which no. histories completed is displayed
      NOUTP = 100

C--- Display opening screen
      WRITE(*,50)
      WRITE(*,51)
      PAUSE '---> Press ENTER for the Main Menu '

C>>>>> The IFLAG variable controls the flow of the program.  If running
C>>>>> on a system which allows for "background" applications.  Then
C>>>>> the flag can be set to 20 and the program will run using the
C>>>>> default file 'mclbrf.inp' as an input file and 'mclbrf.out' as
C>>>>> an output file.
 95   IFLAG=0

C-- Display Main Menu
 96   CALL MENU(IFLAG,IMENU)

      IF (IMENU.EQ.1) THEN
        CALL INP(NPTS,NDIST,NTYPE,MA,EDGEIN,ACOLL,RHOACT)
        CALL INTIAL()
        IFLAG=1
        GOTO 96
      ELSEIF (IMENU.EQ.2) THEN
        WRITE(*,*) ' '
        WRITE(*,FD) 'Enter file name containing problem data ==> '
        READ(*,98) FILEIN
        WRITE(*,*) ' '
        CALL INFILE(NPTS,NDIST,NTYPE,MA,FILEIN,EDGEIN,ACOLL,RHOACT)
        CALL INTIAL()
        IFLAG=1
        GOTO 96
      ELSEIF (IMENU.EQ.3) THEN
        WRITE(*,*) ' '
        WRITE(*,FD) 'Enter file name to save problem data in ==> '
        READ(*,98) FILEOT
        CALL PBDOUT(NPTS,NDIST,NTYPE,NBE,RHOACT,ACOLL,EDGEIN,FILEOT)
        GOTO 96
      ELSEIF (IMENU.EQ.4) THEN
        WRITE(*,*) ' '
        CALL PBDSCR(NPTS,NDIST,NTYPE,NBE,ACOLL,RHOACT,EDGEIN)
        PAUSE '---> Press ENTER to return to Menu'
        GOTO 96
      ELSEIF (IMENU.EQ.5) THEN
        CALL MCARLO(NPTS,NDIST,NTYPE,MA,ACOLL,RHOACT)
        IFLAG=2
        GOTO 96
      ELSEIF (IMENU.EQ.6) THEN
        CALL XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,6)
        PAUSE '---> Press ENTER to return to Menu'
        GOTO 96
      ELSEIF (IMENU.EQ.7) THEN
        WRITE(*,FD) 'Enter file name to save results to ==> '
        READ(*,98) FILEOT
        OPEN(13,FILE=FILEOT,STATUS='UNKNOWN')
        CALL XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,13)
        CLOSE(13)
        IFLAG = 3
        GOTO 96
      ELSEIF (IMENU.EQ.8) THEN
        IF (IFLAG.EQ.2) THEN
          WRITE(*,FD) 'Save results to file? (y/n) ==> '
          READ(*,98) ERFLG
          IF ((ERFLG.EQ.'Y').OR.(ERFLG.EQ.'y')) THEN
            WRITE(*,*) ' '
            WRITE(*,FD) 'Enter file name to save results to ==> '
            READ(*,98) FILEOT
            OPEN(13,FILE=FILEOT,STATUS='UNKNOWN')
            CALL XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,13)
            CLOSE(13)
          END IF
        END IF
        WRITE(*,FD) 'Restart the program? (y/n) ==> '
        READ(*,98) ERFLG
        IF ((ERFLG.EQ.'Y').OR.(ERFLG.EQ.'y')) GOTO 95
        GOTO 96
      ELSEIF (IMENU.EQ.9) THEN
        IF (IFLAG.EQ.2) THEN
          WRITE(*,FD) 'Save results to file? (y/n) ==> '
          READ(*,98) ERFLG
          IF ((ERFLG.EQ.'Y').OR.(ERFLG.EQ.'y')) THEN
            WRITE(*,*) ' '
            WRITE(*,FD) 'Enter file name to save results to ==> '
            READ(*,98) FILEOT
            OPEN(13,FILE=FILEOT,STATUS='UNKNOWN')
            CALL XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,13)
            CLOSE(13)
          END IF
        END IF
        WRITE(*,FD) 'Do you really want to end the program? (y/n) ==> '
        READ(*,98) ERFLG
        IF ((ERFLG.EQ.'Y').OR.(ERFLG.EQ.'y')) GOTO 97
        GOTO 96
      ELSEIF (IMENU.EQ.20) THEN
        FILEIN='mcsky.inp'
        CALL INFILE(NPTS,NDIST,NTYPE,MA,FILEIN,EDGEIN,ACOLL,RHOACT)
        CALL INTIAL()
        CALL MCARLO(NPTS,NDIST,NTYPE,MA,ACOLL,RHOACT)
        OPEN(16,FILE='mcsky.out',STATUS='UNKNOWN')
        CALL XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,16)
        CLOSE(16)
        CALL XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,6)
      END IF

 97    WRITE(*,*) 'Program MCSKY terminate'


  98  FORMAT(A)
  50  FORMAT(24(/),2(25X,11('*'),/),25X,'** McSKY **',/
     1       2(25X,11('*'),/))
  51  FORMAT(
     1' This program calculates the skyshine dose from an isotropic,',/
     2' monoenergetic, point, gamma ray source.    The source can be',/
     3' shielded or unshielded  and the collimation can be  either a',/
     4' silo or a convex polygon.   The program  uses a  Monte Carlo',/
     5' approach to track the gamma rays  through the shield and the',/
     6' approximate  line-beam  response function  to transport them',/
     7' through the atmosphere.',////////)

      END


      SUBROUTINE MENU(IFLAG,IMENU)
C***********************************************************************
C  Purpose:                                                            *
C    This subroutine generates the option menu used to display the     *
C    selections available for running the program.                     *
C                                                                      *
C  Input:                                                              *
C     IFLAG = toggle for foreground and background job                 *
C                                                                      *
C  Output:                                                             *
C     IMENU = option chosen from menu                                  *
C***********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

  80    FORMAT(27(/))
  81    FORMAT(18X,'*************** MCSKY Menu *****************',//
     1         18X,'  1 - Enter problem data from the keyboard  ',/
     2         18X,'  2 - Read problem data from a file         ',//
     3         18X,'  3 - Save problem data to a file           ',/
     4         18X,'  4 - View problem data on the screen       ',//
     5         18X,'  5 - Run the problem                       ',//
     6         18X,'  6 - View results on the screen            ',/
     7         18X,'  7 - Save results to a file                ',//
     8         18X,'  8 - Restart program (i.e. do new case)    ',/
     9         18X,'  9 - Exit program                          ',//
     9         18X,'********************************************',/
     9         18X,'        ENTER an option number (1-9)        ',///)


        IF(IFLAG.EQ.20) THEN
          IMENU=20
          GOTO 99
        END IF

  92    WRITE(*,80)
        WRITE(*,81)
        READ(*,*,ERR=92) IMENU

        IF(IFLAG.EQ.0) THEN
          IF( (IMENU.LT.1).OR.(IMENU.GT.9) ) GOTO 92
          IF( (IMENU.GE.3).AND.(IMENU.LE.7) ) GOTO 92
        ELSEIF(IFLAG.EQ.1) THEN
          IF( (IMENU.LT.1).OR.(IMENU.GT.9) ) GOTO 92
          IF(IMENU.LE.2) GOTO 92
          IF( (IMENU.GE.6).AND.(IMENU.LE.7) ) GOTO 92
        ELSEIF(IFLAG.EQ.2) THEN
          IF( (IMENU.LT.1).OR.(IMENU.GT.9) ) GOTO 92
          IF((IMENU.LE.2).OR.(IMENU.EQ.5)) GOTO 92
        ELSEIF(IFLAG.EQ.3) THEN
          IF( (IMENU.LT.1).OR.(IMENU.GT.9) ) GOTO 92
          IF((IMENU.LE.2).OR.(IMENU.EQ.5)) GOTO 92
        END IF

 99     CONTINUE
      END


      SUBROUTINE INP(NPTS,NDIST,NTYPE,MA,EDGEIN,ACOLL,RHOACT)
C**********************************************************************
C  Purpose:
C    This subroutine asks for input data.
C
C  Input variables:
C    NPTS   = total number of particle histories
C    NDIST  = number of intermediate calculational distances
C    NTYPE  = toggle between a silo or irregular geometry
C    MA     = material number for shields
C    EDGEIN = input file name for irregular shield points
C    ACOLL  = collimation half angle for silo geometry (degrees)
C    RHOACT = air density (g cm^-3)
C
C  Other variables:
C    ENERGY = source photon energy (MeV)
C    HDET   = detector elevation (m)
C    HSLAB  = shield elevation (distance to top of silo) (m)
C    XSTOD  = horizontal distance from source to detector (m)
C    T()    = shield thickness(es)
C
C  Preset values needed:
C    NBE    = number of energy bins
C    NBA    = number of angle bins
C    NBATCH = number of batches (particles per batch = NPTS/NBATCH)
C    DENS() = densities of the four shield materials
C**********************************************************************

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*32 EDGEIN,FD
      COMMON /SENG/ ENERGY
      COMMON /NTARG/ NEDGE,NOUTP
      COMMON /TARGET/ XEDGE(101),YEDGE(101)
      COMMON /BINS/ NBE,NBA,NBATCH
      COMMON /LIMIT/ ELIM(51),PHILIM(21)
      COMMON /PHIX/ XPHI(20)
      COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
      COMMON /DISTMA/ MAT(2)
      COMMON /DENSTY/ DENS(4)

      FD = '(1X,A,$)'

C**** DEFINE THE NUMBER OF BATCHES IN WHICH THE TOTAL NUMBER OF
C     PARTICLES IS DIVIDED

       WRITE (*,FD) 'Enter the total number of particle histories to be
     1tracked ==> '
       READ (*,*) NPTS
       WRITE (*,*) ' '

       WRITE(*,FD) 'Enter the source photon energy (MeV) ==> '
       READ(*,*) ENERGY
       IF (ENERGY.LT.0.02D0) ENERGY=1.25D0
       IF (ENERGY.GT.100.0D0) ENERGY=100.0D0
       WRITE(*,*) ' '

       WRITE(*,FD) 'Enter the desired number of energy bins (<=50) ==> '
       READ(*,*) NBE
       IF (NBE.GT.50) NBE=50
       IF (NBE.LT.1) NBE=1
       XDE=(ENERGY-0.02D0)/NBE
       ELIM(1)=0.020D0
       ELIM(NBE+1)=ENERGY
       DO 1233 I=2,NBE
         ELIM(I)=0.020D0 + 1.D0*(I-1)*XDE
 1233  CONTINUE
       WRITE(*,*) ' '


 1234  WRITE(*,51)
   51  FORMAT (' Select the first (lower) shield material',/
     1         '    0 = No shield',/
     2         '    1 = Aluminum ',/
     3         '    2 = Concrete ',/
     4         '    3 = Iron     ',/
     5         '    4 = Lead     ')
       WRITE(*,FD) 'Enter material number ==> '
       READ(*,*) MAT(1)
       IF( (MAT(1).LT.0).OR.(MAT(1).GT.4) ) GOTO 1234
       WRITE(*,*) ' '

       IF(MAT(1).EQ.0) THEN
         T(1) = 0.0D0
         GOTO 1237
       END IF
       WRITE(*,*) 'Enter the thickness of the first (lower) shield'
       WRITE(*,FD) '(positive number for cm; negative for mfp) ==> '
       READ (*,*) T(1)
       WRITE (*,*) ' '

 1237  WRITE(*,52)
   52  FORMAT (' Select the second (top) shield material',/
     1         '    0 = No shield',/
     2         '    1 = Aluminum ',/
     3         '    2 = Concrete ',/
     4         '    3 = Iron     ',/
     5         '    4 = Lead     ')
       WRITE(*,FD) 'Enter material number ==> '
       READ(*,*) MAT(2)
       IF( (MAT(2).LT.0).OR.(MAT(2).GT.4) ) GOTO 1237
       WRITE(*,*) ' '

       IF(MAT(2).EQ.0) THEN
         T(2) = 0.0
       ELSE
       WRITE(*,*) 'Enter the thickness of the second (top) shield'
       WRITE(*,FD) '(positive number for cm; negative for mfp) ==> '
         READ(*,*) T(2)
         WRITE(*,*) ' '
       END IF

       IF((MAT(1).EQ.0).AND.(MAT(2).EQ.0)) THEN
         MA = 2
       ELSEIF(MAT(1).EQ.0) THEN
         MA = MAT(2)
       ELSE
         MA = MAT(1)
       END IF

       WRITE(*,FD) 'Enter the air mass density (g/cm^3) ==> '
       READ(*,*) RHOACT
       WRITE(*,*) ' '

       WRITE(*,53)
   53  FORMAT(' Enter the vertical distance between the lower shield and
     1 the',/' horizontal plane through the source (m) ',/
     2 ' (NOTE: this must be a positive number) ==> ',$)
       READ (*,*) HSLAB
       HSL=HSLAB*100.0D0
       WRITE (*,*) ' '

       WRITE(*,54)
   54  FORMAT(' Enter the vertical distance between the horizontal plane
     1 through',/' the detector and the horizontal plane through the sou
     2rce (m) ',/
     3' (NOTE: distance is positive for detectors located above the sour
     4ce',/'        and negative for detectors below the source elevatio
     5n) ==> ',$)
       READ (*,*) HDET
       WRITE (*,*) ' '

       WRITE (*,FD) 'Enter the horizontal distance from the source to de
     1tector (m) ==> '
       READ (*,*) XSTOD
       WRITE (*,*) ' '

       WRITE (*,*) 'Enter the number of intermediate distances at which
     1the skyshine dose'
       WRITE (*,FD) 'should be calculated ( <=19 ) ==> '
       READ (*,*) NDIST
       WRITE (*,*) ' '
       IF (NDIST.GT.19) NDIST=19
       XMARK = XSTOD/(NDIST+1)
       DO 1238 M=1,NDIST+1
         XPHI(M) = 1.0D0*M*XMARK
 1238  CONTINUE

 1235  WRITE (*,*) 'Select the geometry of the source collimation'
       WRITE (*,*) '   1 = Silo collimation'
       WRITE (*,*) '   2 = Convex N-point polygon collimation'
       WRITE (*,FD) 'Enter the source collimation number ==> '
       READ (*,*) NTYPE
       IF( (NTYPE.LT.1).OR.(NTYPE.GT.2) ) GOTO 1235
       WRITE(*,*) ' '

       IF (NTYPE.EQ.1) THEN
 1236    WRITE(*,FD) 'Enter the half-angle of collimation in degrees (0<
     1angle<90) ==> '
         READ(*,*) ACOLL
         WRITE(*,*) ' '
         IF(ACOLL.GT.180.0) GOTO 1236
       ELSE
         WRITE(*,FD)  'Enter the file name containing the N-point polygon
     1 vertices ==> '
         READ (*,'(A)') EDGEIN
         OPEN(9,FILE=EDGEIN,STATUS='UNKNOWN')
         READ(9,*) NEDGE
         IF(NEDGE.GT.100) NEDGE=100
         DO 220 I=1,NEDGE
           READ(9,*) XEDGE(I),YEDGE(I)
 220     CONTINUE
         XEDGE(NEDGE+1) = XEDGE(1)
         YEDGE(NEDGE+1) = YEDGE(1)
         CLOSE(9)
         RCMAX = DSQRT(XEDGE(1)*XEDGE(1)+YEDGE(1)*YEDGE(1))
         DO 221 I=2, NEDGE
           RC = DSQRT(XEDGE(I)*XEDGE(I)+YEDGE(I)*YEDGE(I))
           IF (RC.GT.RCMAX) RCMAX=RC
 221     CONTINUE
         ACOLLR = DACOS(HSLAB/DSQRT(HSLAB*HSLAB+RCMAX*RCMAX))
         ACOLL = 180.D0*ACOLLR/3.141592653590
       END IF


        IF (T(1).LT.0.0) THEN
          T1 = -T(1)/(TOT(DLOG10(ENERGY),MAT(1))*DENS(MAT(1)))
        ELSE
          T1 = T(1)
        END IF
        IF (T(2).LT.0.0) THEN
          T2 = -T(2)/(TOT(DLOG10(ENERGY),MAT(2))*DENS(MAT(2)))
        ELSE
          T2 = T(2)
        END IF

        CONTINUE
      END


      SUBROUTINE INFILE(NPTS,NDIST,NTYPE,MA,FILEIN,EDGEIN,ACOLL,RHOACT)
C**********************************************************************
C  Purpose:
C    This subroutine retrieves input data from a predefined file.
C
C  Input variables:
C    NPTS   = total number of particle histories
C    NDIST  = number of intermediate calculational distances
C    NTYPE  = toggle between a silo or irregular geometry
C    MA     = material number for shields
C    EDGEIN = input file name for irregular shield points
C    ACOLL  = collimation half angle for silo geometry (degrees)
C    RHOACT = air density (g cm^-3)
C
C  Other variables:
C    ENERGY = source photon energy (MeV)
C    HDET   = detector elevation (m)
C    HSLAB  = shield elevation (distance to top of silo) (m)
C    XSTOD  = horizontal distance from source to detector (m)
C    T()    = shield thickness(es)
C
C  Preset values needed:
C    NBE    = number of energy bins
C    NBA    = number of angle bins
C    NBATCH = number of batches (particles per batch = NPTS/NBATCH)
C    DENS() = densities of the four shield materials
C......................................................................

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        CHARACTER*32 FILEIN,EDGEIN

        COMMON /SENG/ ENERGY
        COMMON /NTARG/ NEDGE,NOUTP
        COMMON /TARGET/ XEDGE(101),YEDGE(101)
        COMMON /BINS/NBE,NBA,NBATCH
        COMMON /LIMIT/ ELIM(51),PHILIM(21)
        COMMON /PHIX/ XPHI(20)
        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /DISTMA/ MAT(2)
        COMMON /DENSTY/ DENS(4)

        OPEN(11,FILE=FILEIN,STATUS='UNKNOWN')

        READ(11,*) NPTS
        READ(11,*) NBE
        READ(11,*) ENERGY
        READ(11,*) MAT(1),MAT(2)
        READ(11,*) T(1),T(2)
        READ(11,*) RHOACT
        READ(11,*) HSLAB,HDET
        READ(11,*) XSTOD
        READ(11,*) NDIST
        READ(11,*) NTYPE
        IF(NTYPE.EQ.1) THEN
          READ(11,*) ACOLL
          CLOSE(11)
        ELSE
          READ(11,'(A)') EDGEIN
          CLOSE(11)
          ACOLL = 90.0D0
          OPEN(12,FILE=EDGEIN,STATUS='UNKNOWN')
          READ(12,*) NEDGE
          IF(NEDGE.GT.100) NEDGE=100
          DO 210 I=1, NEDGE
            READ(12,*) XEDGE(I),YEDGE(I)
 210      CONTINUE
          CLOSE(12)
          XEDGE(NEDGE+1)=XEDGE(1)
          YEDGE(NEDGE+1)=YEDGE(1)
          RCMAX = DSQRT(XEDGE(1)*XEDGE(1)+YEDGE(1)*YEDGE(1))
          DO 221 I=2, NEDGE
            RC = DSQRT(XEDGE(I)*XEDGE(I)+YEDGE(I)*YEDGE(I))
            IF (RC.GT.RCMAX) RCMAX=RC
 221      CONTINUE
          ACOLLR = DACOS(HSLAB/DSQRT(HSLAB*HSLAB+RCMAX*RCMAX))
          ACOLL = 180.D0*ACOLLR/3.141592653590
        END IF

        IF(NDIST.GT.19) NDIST=19

        HSL=HSLAB*100.0D0

        IF (NBE.LT.1) NBE=1
        XDE=(ENERGY-0.02D0)/NBE
        ELIM(1)=0.020D0
        ELIM(NBE+1) = ENERGY
        DO 212 I=2,NBE
          ELIM(I)=0.020D0 + 1.D0*(I-1)*XDE
 212    CONTINUE

        IF((MAT(1).EQ.0).AND.(MAT(2).EQ.0)) THEN
          MA = 2
        ELSEIF(MAT(1).EQ.0) THEN
          MA = MAT(2)
        ELSE
          MA = MAT(1)
        END IF

        IF (T(1).LT.0.0) THEN
          T1 = -T(1)/(TOT(DLOG10(ENERGY),MAT(1))*DENS(MAT(1)))
        ELSE
          T1 = T(1)
        END IF
        IF (T(2).LT.0.0) THEN
          T2 = -T(2)/(TOT(DLOG10(ENERGY),MAT(2))*DENS(MAT(2)))
        ELSE
          T2 = T(2)
        END IF

        XMARK = XSTOD/(NDIST+1)
        DO 211 M=1,NDIST+1
          XPHI(M) = 1.0D0*M*XMARK
 211    CONTINUE
      END



      SUBROUTINE PBDSCR(NPTS,NDIST,NTYPE,NBE,ACOLL,RHOACT,EDGEIN)
C**********************************************************************
C  Purpose:
C    This subroutine displays problem data to the screen.
C
C  Variables displayed:
C    NPTS   = total number of particle histories
C    NBE    = number of energy bins used
C    ENERGY = source photon energy (MeV)
C    RHOACT = air density (g cm^-3)
C    MAT()  = shield number (i.e. shield material)
C    T()    = shield thickness
C    HSLAB  = shield height from ground (m)
C    HDET   = detector height from ground (m)
C    XSTOD  = horizontal distance from source to detector (m)
C    NDIST  = number of intermediate calculational distances
C    ACOLL  = collimation angle or a message for irregular geometry
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        CHARACTER*32 EDGEIN

        COMMON /SENG/ ENERGY
        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /DISTMA/ MAT(2)

 811    FORMAT( ' Total number of particle histories = ',I10,/)
        WRITE(*,811) NPTS

 812    FORMAT( ' Number of energy bins used = ',I3)
        WRITE(*,812) NBE

 813    FORMAT( ' Source photon energy (MeV) =',F10.3)
        WRITE(*,813) ENERGY

 814    FORMAT(' Air density (g/cm^3) = ',F10.6,/)
        WRITE(*,814) RHOACT

        IF(MAT(1).EQ.0) THEN
          WRITE(*,*) 'No lower (1st) shield'
          WRITE(*,*) ' '
          GOTO 822
        ELSEIF(MAT(1).EQ.1) THEN
          WRITE(*,*) 'Lower shield material = ALUMINUM'
        ELSEIF(MAT(1).EQ.2) THEN
          WRITE(*,*) 'Lower shield material = CONCRETE'
        ELSEIF(MAT(1).EQ.3) THEN
          WRITE(*,*) 'Lower shield material = IRON'
        ELSE
          WRITE(*,*) 'Lower shield material = LEAD'
        END IF

 815    FORMAT(' Shield thickness (mfp) = ',F8.2)
 816    FORMAT(' Shield thickness (cm)  = ',F8.2)
        IF (T(1).LT.0.0) THEN
          WRITE(*,815) -T(1)
        ELSE
          WRITE(*,816) T(1)
        END IF

 822    IF(MAT(2).EQ.0) THEN
          WRITE(*,*) 'No upper (2nd) shield.'
          GOTO 823
        ELSEIF(MAT(2).EQ.1) THEN
          WRITE(*,*) 'Upper shield material = ALUMINUM'
        ELSEIF(MAT(2).EQ.2) THEN
          WRITE(*,*) 'Upper shield material = CONCRETE'
        ELSEIF(MAT(2).EQ.3) THEN
          WRITE(*,*) 'Upper shield material = IRON'
        ELSE
          WRITE(*,*) 'Upper shield material = LEAD'
        END IF

        IF (T(2).LT.0.0) THEN
          WRITE(*,815) -T(2)
        ELSE
          WRITE(*,816) T(2)
        END IF
 823    WRITE(*,*) ' '

 817    FORMAT( ' Source to lower shield distance (m) = ',F8.2)
        WRITE(*,817) HSLAB

 818    FORMAT( ' Detector height above source elevation (m) = ',
     *  F8.2,/)
        WRITE(*,818) HDET

 819    FORMAT( ' Max. source-to-detector distance (m) = ',F10.2)
        WRITE(*,819) XSTOD

 820    FORMAT( ' No. of intermediate calculational distances = ',I3,/)
        WRITE(*,820) NDIST

 821    FORMAT( ' collimation half-angle (deg) = ',F8.2,/)
        IF(NTYPE.EQ.1) THEN
          WRITE(*,*) 'Source collimation = SILO GEOMETRY'
          WRITE(*,821) ACOLL
        ELSE
          WRITE(*,*) 'Source collimation = CONVEX N-POINT GEOMETRY'
          WRITE(*,*) '    (using vertices in file ',EDGEIN,')'
          WRITE(*,*) ' '
        END IF

      END


      SUBROUTINE INTIAL()
C**********************************************************************
C  Purpose:
C    This subroutine initializes or resets the bin tallies to zero.
C
C  Variables:
C    NBE    = number of energy bins
C    NBA    = number of angle bins
C    NBATCH = number of batches
C    IEPHI   = collided escaping photon energy-angle bin
C    IPPPHI  = uncollided pair production angle bin
C    ISPHI   = uncollided source particle angle bin
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /BINS/ NBE,NBA,NBATCH
        COMMON /BINEP/ IEPHI(20,50,20)
        COMMON /PPBIN/ IPPPHI(20,20)
        COMMON /MNBIN/ ISPHI(20,20)

        DO 16 M=1, 20
        DO 12 I=1, NBA
          DO 10 J=1, NBE
            IEPHI(M,J,I) = 0.0D0
  10      CONTINUE
          IPPPHI(M,I) = 0.0D0
          ISPHI(M,I) = 0.0D0
  12    CONTINUE
  16    CONTINUE

      END


      SUBROUTINE PBDOUT(NPTS,NDIST,NTYPE,NBE,RHOACT,ACOLL,EDGEIN,FILEOT)
C************************************************************************
C  Purpose:
C    This subroutine outputs the problem data to a file for possible
C    later use.  The data is in the format needed for later input.
C
C  Output variables:
C    NPTS   = total number of particle histories
C    NDIST  = number of intermediate calculational distances
C    NTYPE  = selection option for silo collimation/irregular shield
C    NBE    = number of energy bins used
C    RHOACT = air density (g cm^-3)
C    ACOLL  = source maximum collimation angle
C    EDGEIN = input file name for irregular shield points
C    FILEOT = name of file to be written to
C
C  Other output variables:
C    ENERGY = source photon energy (MeV)
C    MAT()  = shield number (i.e. shield material)
C    T()    = shield thickness
C    HSLAB  = shield height from ground (m)
C    HDET   = detector height from ground (m)
C    XSTOD  = horizontal distance from source to detector (m)
C************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        CHARACTER*32 FILEOT,EDGEIN

        COMMON /SENG/ ENERGY
        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /DISTMA/ MAT(2)

        OPEN(12,FILE=FILEOT,STATUS='UNKNOWN')

        WRITE(12,21) NPTS
        WRITE(12,21) NBE
        WRITE(12,22) ENERGY
        WRITE(12,23) MAT(1),MAT(2)
        WRITE(12,29) T(1),T(2)
        WRITE(12,24) RHOACT
        WRITE(12,25) HSLAB,HDET
        WRITE(12,26) XSTOD
        WRITE(12,27) NDIST
        WRITE(12,28) NTYPE
        IF(NTYPE.EQ.1) THEN
          WRITE(12,30) ACOLL
        ELSE
          WRITE(12,31) EDGEIN
        END IF

        CLOSE(12)
 21     FORMAT(I15)
 22     FORMAT(F10.3)
 23     FORMAT(I2,8X,I2)
 24     FORMAT(G12.4)
 25     FORMAT(F10.2,1X,F10.2)
 26     FORMAT(F10.2)
 27     FORMAT(I3)
 28     FORMAT(I2)
 29     FORMAT(F10.2,1X,F10.2)
 30     FORMAT(F10.2)
 31     FORMAT(72A)

      END



      SUBROUTINE MCARLO(NPTS,NDIST,NTYPE,MA,ACOLL,RHOACT)
C************************************************************************
C  Purpose:
C    This subroutine (1) inputs the necessary data for the approximate
C    line beam, (2) calls the necessary subroutines to track photon
C    through the shield, (3) normalizes the bin tallies, and (4) calls
C    for the skyshine dose.
C
C  Input Arguments:
C    NPTS   = total number of particle histories
C    NDIST  = number of intermediate calculational distances
C    NTYPE  = selection option for silo collimation/irregular shield
C    MA     = shield material number
C    ACOLL  = maximum source collimation angle
C    RHOACT = air density (g cm^-3)
C
C  Other subroutine variables:
C    ENERGY = source photon energy (MeV)
C    MAT()  = shield number (i.e. shield material)
C    T()    = shield thickness
C    HSLAB  = shield height from ground (m)
C    HDET   = detector height from ground (m)
C    XSTOD  = horizontal distance from source to detector (m)
C    NBE    = number of energy bins
C    NBA    = number of angle bins
C    NBATCH = number of batches
C
C  Output (in COMMON/BATCH/):
C    SUM1 = Vector of skyshine dose for each distance
C    SUM2 = Vector of the square of skyshine dose for each distance
C           (Needed for statistical purposes)
C
C  NEEDED SUBROUTINES:
C    SOURCE = source photon direction generation subroutine
C    SHIELD = controls Monte Carlo walk of photon through shield
C    XBEAM  = Approximate 3-parameter LBRF
C     or
C    XBEAM4 = Approximate 4-parameter LBRF
C************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /SENG/ ENERGY
        COMMON /NTARG/ NEDGE,NOUTP
        COMMON /TARGET/ XEDGE(101),YEDGE(101)
        COMMON /BINS/ NBE,NBA,NBATCH
        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /DISTMA/ MAT(2)
        COMMON /RESPNC/ RESPEP,RESPPP,RESPMN
        COMMON /PHIX/ XPHI(20)
        COMMON /BINEP/ IEPHI(20,50,20)
        COMMON /PPBIN/ IPPPHI(20,20)
        COMMON /MNBIN/ ISPHI(20,20)
        COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)  !<<<<<<<<< Revised Oct 97, REF
        COMMON /DENSTY/ DENS(4)
        COMMON /LIMIT/ ELIM(51),PHILIM(21)
        COMMON /BATCH/ SUM1(20),SUM2(20)
        COMMON /PARKD/ SEED

C** Set variables used in subroutine
        SEED=1.0D0
        ACOLL1 = DCOS(ACOLL*3.141592/180.0)
        NPTB = NPTS/NBATCH
        XNTOT = (1.D0-ACOLL1)/2.
        XPI = DACOS(-1.0D0)
        T12 = (T1+T2)/100.0D0

C** Set skyshine dose and statistic vectors to zero
        DO 13 I=1, NDIST+1
          SUM1(I) = 0.0D0
          SUM2(I) = 0.0D0
 13     CONTINUE

C>>>>>>> The term IPARM controls which line-beam response function
C>>>>>>> used.  If IPARM=4 then the four parameter line-beam
C>>>>>>> approximation is used.  For any other value the three
C>>>>>>> parameter line-beam approximation is used (although
C>>>>>>> IPARM=3 would make the most sense).
        IPARM=3

C** Read in LBRF approximation parameters
        CALL XBINI(IPARM)

C---- Track photon through shield
        WRITE (*,'(24(/))')
        WRITE (*,*) 'TRACKING PARTICLES THROUGH THE SHIELDS . . . '
        WRITE (*,*) ' '

        DO 14 L=1,NBATCH
          CALL INTIAL()
          NTOT = 0
          DO 15 K=1, NPTB

            CALL SOURCE(Z1,W1,P1,XENG,HSLAB,NTOT,NTYPE,ACOLL1)
            CALL SHIELD(NDIST,NBE,NBA,MA,XENG,Z1,W1,P1)

            KLNPT = K+(L-1)*NPTB
            IF(MOD(KLNPT,NOUTP).EQ.0) THEN
              WRITE(*,42) KLNPT,NPTS,100.D0*KLNPT/NPTS
            END IF

   15     CONTINUE

          FCORR = XNTOT/NTOT

C---- Use generated binned scores for this batch to calculate skyshine dose
          DO 41 IJK = 1,NDIST+1
            XSTD1 = XPHI(IJK)
            RBATCH = 0.0D0
            RESPEP = 0.0D0
            RESPMN = 0.0D0
            RESPPP = 0.0D0
            XSD = DSQRT(XSTD1*XSTD1 + (HDET-T12-HSLAB)**2)
            DO 900 J=1, NBA
              ANGLE = (PHILIM(J)+PHILIM(J+1))/2.0D0

C-- Annihilation component
              E = 0.511D0
              CALL XBEAM(E,ANGLE,XSD,RHOACT,RESP,IPARM)
              RESPPP = RESPPP + FCORR*IPPPHI(IJK,J)*RESP

C-- Uncollided source photon component
              E = ENERGY
              CALL XBEAM(E,ANGLE,XSD,RHOACT,RESP,IPARM)
              RESPMN = RESPMN + FCORR*ISPHI(IJK,J)*RESP

C-- Collided escaping photons
              DO 910 KK=1,NBE
                E = (ELIM(KK)+ELIM(KK+1))/2.0D0
                CALL XBEAM(E,ANGLE,XSD,RHOACT,RESP,IPARM)
                RESPEP = RESPEP + FCORR*IEPHI(IJK,KK,J)*RESP
 910          CONTINUE
 900        CONTINUE

C----- Store skyshine dose for this batch in the SUM vectors
            RBATCH = RESPEP+RESPPP+RESPMN
            SUM1(IJK) = SUM1(IJK) + RBATCH
            SUM2(IJK) = SUM2(IJK) + RBATCH*RBATCH

   41     CONTINUE
   14   CONTINUE

   42   FORMAT('+',I12,' particles of ',I12,
     *  '; Percent completed = ',F7.1,' %')

        RETURN
      END



      SUBROUTINE BIN(NBE,NBA,M,WPHI,XENG)
C***********************************************************************
C  Purpose:
C    This subroutine determines which energy-angle bin to augemnt in the
C    IEPHI tally array for an escaping collided photon. The appropriate
C    bin is augmented by 1.
C
C  Input variables:
C    NBE  = number of energy bins
C    NBA  = number of angle bins
C    M    = distance subscript for detector location
C    WPHI = cosine of source-to-detector escaping angle
C    XENG = energy of escaping photon (MeV)
C
C  Output (in COMMON/BINEP/):
C    IEPHI(M,Nenergy,Nangle) = energy-angle binning array
C
C  Needed variables (in COMMON/LIMIT/):
C    ELIM   = energy limits on energy bins
C    PHILIM = angle limits on angle bins
C***********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /BINEP/ IEPHI(20,50,20)
        COMMON /LIMIT/ ELIM(51),PHILIM(21)

        PHI = DACOS(WPHI)
        PHI = PHI*180.0D0/DACOS(-1.0D0)

        JL = 0
        JU = NBE+2

C*** Find bin for the photon energy
  40    IF (JU-JL.GT.1) THEN
          JM = (JU+JL)/2
          IF ((ELIM(NBE+1).GT.ELIM(1)).EQV.(XENG.GT.ELIM(JM))) THEN
            JL = JM
          ELSE
            JU = JM
          END IF
          GOTO 40
        END IF
        IDXE = JL
        JL = 0
        JU = NBA+2

C*** Find bin for the photon angle
  50    IF (JU-JL.GT.1) THEN
          JM = (JU+JL)/2
          IF ((PHILIM(NBA+1).GT.PHILIM(1)).EQV.(PHI.GT.PHILIM(JM))) THEN
            JL = JM
          ELSE
            JU = JM
          END IF
          GOTO 50
        END IF
        IDXA = JL

        IEPHI(M,IDXE,IDXA) = IEPHI(M,IDXE,IDXA) + 1

      END



      SUBROUTINE BINPP(NBA,M,WPHI)
C************************************************************************
C  Purpose:
C    This subroutine determines which energy-angle bin to augemnt in the
C    IPPPHI tally array for an escaping uncollided pair production photon.
C    The appropriate bin is augmented by 1.
C
C  Input variables:
C    NBA  = number of angle bins
C    M    = distance subscript
C    WPHI = cosine of source-to-detector escaping angle
C
C  Output (in COMMON/PPBIN/):
C    IPPPHI(M,Nangle) = angle binning array
C
C  Needed variables:
C    PHILIM = angle limits on angle bins (in COMMON/LIMIT/)
C************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /PPBIN/ IPPPHI(20,20)
        COMMON /LIMIT/ ELIM(51),PHILIM(21)

        PHI = DACOS(WPHI)
        PHI = PHI*180.0D0/3.14159265358979D0
        JL = 0
        JU = NBA+2

C** Find angle bin index for given direction
  60    IF (JU-JL.GT.1) THEN
          JM = (JU+JL)/2
          IF ((PHILIM(NBA+1).GT.PHILIM(1)).EQV.(PHI.GT.PHILIM(JM))) THEN
            JL = JM
          ELSE
            JU = JM
          END IF
          GOTO 60
        END IF
        IDXA = JL

        IPPPHI(M,IDXA) = IPPPHI(M,IDXA) + 1
      END



      SUBROUTINE BINMN(NBA,M,WPHI)
C**********************************************************************
C  Purpose:
C    This subroutine determines which energy-angle bin to augemnt in the
C    ISPHI tally array for an escaping uncollided source photon. The
C    appropriate bin is augmented by 1.
C
C  Input variables:
C    This subroutine determines which energy-angle bin to increase the
C    tally in for an escaping uncollided source photon.
C
C  Input:
C    NBA  = number of angle bins
C    M    = distance subscript
C    WPHI = cosine of source-to-detector escaping angle
C
C  Output ( COMMON/MNBIN/):
C    ISPHI(M,Nangle) = source photon angle binning array
C
C  Needed variables:
C    PHILIM = angle limits on angle bins (in COMMON/LIMIT/)
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /MNBIN/ ISPHI(20,20)
        COMMON /LIMIT/ ELIM(51),PHILIM(21)

        PHI = DACOS(WPHI)
        PHI = PHI*180.0D0/3.14159265358979D0
        JL = 0
        JU = NBA+2

C** Find index for angle bin
  70    IF (JU-JL.GT.1) THEN
          JM = (JU+JL)/2
          IF ((PHILIM(NBA+1).GT.PHILIM(1)).EQV.(PHI.GT.PHILIM(JM))) THEN
            JL = JM
          ELSE
            JU = JM
          END IF
          GOTO 70
        END IF
        IDXA = JL

        ISPHI(M,IDXA) = ISPHI(M,IDXA) + 1

      END




      SUBROUTINE SOURCE(Z,W,P,XENG,HSLAB,NTOT,NTYPE,ACOLL1)
C***********************************************************************
C  Purpose:
C    This subroutine generates a "random direction" for a source photon
C    that hits the bottom of the source shield within the specified
C    collimation.
C
C  Input variables:
C    HSLAB  = thickness of overhead source shield (m)
C    NTYPE  = source collimation geometry (=1 silo; =2 N-polygon)
C    ACOLL1 = cosine of maximum source collimation angle
C
C  Output variables:
C    Z     = z position (cm) of photon entering the shield (=100*HSLAB)
C    W     = cosine of polar angle of direction
C    P     = azimuthal angle in radians (0 - 2pi)
C    XENG  = energy of photon (MeV)
C    NTOT  = total number of source photons generated for the simulation
C
C  NEEDED SUBROUTINES/FUNCTIONS:
C    FLTRN = random number generating function
C    DIRCOS = calculates direction cosines
C***********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /SENG/ ENERGY
        COMMON /PARKD/ SEED
        COMMON /NTARG/ NEDGE,NOUTP
        COMMON /TARGET/ XEDGE(101),YEDGE(101)

        PARAMETER (TWOPI=6.2831853D0)

C** Generate random emission direction
 100    W = 1.D0 - (1.D0-ACOLL1)*FLTRN()
        P = TWOPI*FLTRN()
        NTOT = NTOT + 1

        IF(NTYPE.EQ.1) THEN
          Z = HSLAB
        ELSE
          Z = HSLAB
          SDUM = Z/W
          CALL DIRCOS(P,U,V,W)
          X = SDUM * U
          Y = SDUM * V
C-- test that photon hits shield within collimation polygon
          TST1 = (XEDGE(2)-XEDGE(1))*(Y-YEDGE(1))-(YEDGE(2)-YEDGE(1))*
     *                                                     (X-XEDGE(1))
          DO 110 I = 2, NEDGE
            TST2 = (XEDGE(I+1)-XEDGE(I))*(Y-YEDGE(I))-(YEDGE(I+1)
     *                                          -YEDGE(I))*(X-XEDGE(I))
            IF ((TST1*TST2).LE.0) THEN
              GOTO 100
            ELSE
              TST1 = TST2
            END IF
 110      CONTINUE
        END IF

        Z = Z*100.0D0
        XENG = ENERGY

        RETURN
      END







      SUBROUTINE CMPT(XLAM1,XLAM2)
C***********************************************************************
C  Purpose:
C    Determines the energy loss in terms of Compton wavelengths due to
C    a Compton scatter.
C
C  Input:
C    XLAM1 = initial Compton wavelength
C
C  Output:
C    XLAM2 = scattered Compton wavelength
C
C  NEEDED FUNCTION:
C    FLTRN = random number generator
C***********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /PARKD/ SEED

  600   RHO1 = FLTRN()
        RHO2 = FLTRN()
        RHO3 = FLTRN()

        IF (RHO1.LT.(XLAM1+2.0D0)/(9.0D0*XLAM1+2.0D0)) THEN
          R = 2.0D0*RHO2/XLAM1 + 1.0D0
          IF (RHO3.LT.4.0D0*(1./R - 1./R/R)) THEN
            GOTO 610
          ELSE
            GOTO 600
          END IF
        ELSE
          R = (XLAM1+2.0D0)/(2.0D0*RHO2 + XLAM1)
          IF (RHO3.LT.0.5D0*(1/R+(XLAM1+1.0D0-R*XLAM1)**2)) THEN
            GOTO 610
          ELSE
            GOTO 600
          END IF
        END IF

 610    XLAM2 = R*XLAM1

        RETURN
      END






      SUBROUTINE SHIFT(W1,P1,COSS,W2,P2)
C**********************************************************************
C  Purpose:
C    Calculates new cosine of azimuthal angle and new polar angle
C    after a Compton scatter.
C
C  Input:
C    W1   = cosine of incident polar angle
C    P1   = incident azimuthal angle (radians)
C    COSS = cosine of the scattering angle
C
C  Output:
C    W2 = cosine of emergent polar angle
C    P1 = emergent azimuthal angle (radians)
C
C  NEEDED FUNCTIONS:
C    FLTRN = random number generator
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /PARKD/ SEED
        PARAMETER (PI=3.14159265358979,TWOPI=6.28318530717959)

C--- Determine azimuthal shift
        PS = TWOPI*FLTRN()

C--- Determine cosine of emergent polar angle
        SS = DSQRT(1.0D0-COSS*COSS)
        S1 = DSQRT(1.0D0-W1*W1)
        W2 = W1*COSS + SS*S1*DCOS(PS)
        S2 = DSQRT(1.0D0-W2*W2)

C--- Determine emergent azimuthal angle
C       P2 = DABS(P1 - DACOS((COSS-W1*W2)/S1/S2))
        P2 = P1 + DACOS((COSS-W1*W2)/S1/S2)
        P2 = MOD(P2,TWOPI)

        RETURN
      END



      SUBROUTINE DIRCOS(P,U,V,W)
C**********************************************************************
C  Purpose:
C    Calculate the direction cosines for the x and y axis.
C
C  Input:
C    P = azimuthal angle (radians)
C    W = cosine of the polar angle
C
C  Output
C    U = direction cosine for the x axis
C    V = direction cosine for the y axis
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        SDIR = DSQRT(1.0-W*W)
        U = DCOS(P)*SDIR
        V = DSIN(P)*SDIR
        RETURN
      END



      DOUBLE PRECISION FUNCTION PATHL(XMU,MA)
C**********************************************************************
C  Purpose:
C    Calculate a path length for photon travel. (cm)
C
C  Input:
C    XMU = total interaction cross section for photon (cm^2/g)
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /DENSTY/ DENS(4)
      COMMON /PARKD/ SEED
        PATHL = -DLOG(FLTRN())/(XMU*DENS(MA))
        RETURN
      END



      DOUBLE PRECISION FUNCTION COM(X,MA)
C**********************************************************************
C  Purpose:
C    Determine the Compton scattering cross section for a photon of
C    given energy.
C
C  Input:
C    X = log base ten of the photon energy
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)    !<<<<<<<<< Revised Oct 97, REF
        COM = AMU(MA,1,1) + X*(AMU(MA,1,2) + X*(AMU(MA,1,3)
     *                    + X*(AMU(MA,1,4) + X*(AMU(MA,1,5)
     *                    + X*(AMU(MA,1,6) + X*(AMU(MA,1,7)
     *                    + X*(AMU(MA,1,8) + X*(AMU(MA,1,9)
     *                    + X*(AMU(MA,1,10) + X*AMU(MA,1,11))))))))))
        COM = 10.0D0**COM
        COM = COM*1.0D-24*6.02252D23/AMASS(MA)
        RETURN
      END



      DOUBLE PRECISION FUNCTION PPAIR(X,MA)
C**********************************************************************
C  Purpose:
C    Determine the pair production cross section for a photon of
C    given energy.
C
C  Input:
C    X = log base ten of the photon energy
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)    !<<<<<<<<< Revised Oct 97, REF

        IF (X.LT.(DLOG10(1.022D0))) THEN
          PPAIR = 0.0D0
        ELSE
          PPAIR = AMU(MA,2,1) + X*(AMU(MA,2,2) + X*(AMU(MA,2,4)
     *                        + X*(AMU(MA,2,6) + X*(AMU(MA,2,8)))))
     *                        + ((AMU(MA,2,7)/X + AMU(MA,2,5))/X
     *                        + AMU(MA,2,3))/X
          PPAIR = 10.0D0**PPAIR
          PPAIR = PPAIR*1.0D-24*6.02252D23/AMASS(MA)
        END IF
        RETURN
      END




      DOUBLE PRECISION FUNCTION TOT(X,MA)
C**********************************************************************
C  Purpose:
C    Determine the total interaction cross section for a photon of
C    given energy.
C
C  Input:
C    X = log base ten of the photon energy
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)    !<<<<<<<<< Revised Oct 97, REF
        TOT = AMU(MA,3,1) + X*(AMU(MA,3,2) + X*(AMU(MA,3,3)
     *                    + X*(AMU(MA,3,4) + X*(AMU(MA,3,5)
     *                    + X*(AMU(MA,3,6) + X*(AMU(MA,3,7)
     *                    + X*(AMU(MA,3,8) + X*(AMU(MA,3,9)
     *                    + X*(AMU(MA,3,10) + X*(AMU(MA,3,11)))))))))))
        TOT = 10.0D0**TOT
        RETURN
      END



      DOUBLE PRECISION FUNCTION FLTRN()
C***********************************************************************
C  Purpose:
C    Generate a random number sequence.
C***********************************************************************
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /PARKD/ SEED

        PARAMETER(A=16807.D0,C=2147483647.D0)
        PARAMETER(B=.465661287524579692D-9)
        TEMP = SEED*A
        SEED = TEMP-C*INT(TEMP*B)
        FLTRN = SEED*B

        RETURN
      END



      SUBROUTINE GETWP(W,P)
C**********************************************************************
C  Purpose:
C    Return the cosine of the polar angle and the azimuthal angle
C    for a direction uniformly selected from 4pi steradians.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PARKD/ SEED
        PARAMETER (TWOPI=6.2831853D0)
        W = -1.D0+2.D0*FLTRN()
        P = TWOPI*FLTRN()
        RETURN
      END



      SUBROUTINE XLBRF(NPTS,NBATCH,NDIST,NTYPE,ACOLL,RHOACT,IOUT)
C***********************************************************************
C  Purpose:
C    Output the results of the skyshine dose calculation to the device
C    specified by the IOUT variable.  Generally this is either the
C    screen of a file.
C***********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /SENG/ ENERGY
        COMMON /PHIX/ XPHI(20)
        COMMON /BATCH/ SUM1(20),SUM2(20)
        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /DISTMA/ MAT(2)
        COMMON /NTARG/ NEDGE,NOUTP
        COMMON /TARGET/ XEDGE(101),YEDGE(101)

C****** OUTPUT GENERAL PROBLEM DATA

        IF (IOUT.EQ.6) WRITE(IOUT,'(20(/))')
        EMAX = ENERGY
        WRITE (IOUT,955) NPTS,EMAX,HSLAB,HDET,XSTOD

        IF(MAT(1).EQ.0) THEN
          WRITE(IOUT,*) 'No lower (1st) shield.'
          GOTO 953
        ELSEIF(MAT(1).EQ.1) THEN
          WRITE(IOUT,*) 'Lower (1st) Shield material = ALUMINUM'
        ELSEIF(MAT(1).EQ.2) THEN
          WRITE(IOUT,*) 'Lower (1st) Shield material = CONCRETE'
        ELSEIF(MAT(1).EQ.3) THEN
          WRITE(IOUT,*) 'Lower (1st) Shield material = IRON'
        ELSE
          WRITE(IOUT,*) 'Lower (1st) Shield material = LEAD'
        END IF

        IF (T(1).LT.0.0) THEN
          WRITE(IOUT,957) -T(1)
        ELSE
          WRITE(IOUT,956) T(1)
        END IF

 953    IF(MAT(2).EQ.0) THEN
          WRITE(IOUT,*) 'No upper (2nd) shield.'
          GOTO 958
        ELSEIF(MAT(2).EQ.1) THEN
          WRITE(IOUT,*) 'Upper (2nd) Shield material = ALUMINUM'
        ELSEIF(MAT(2).EQ.2) THEN
          WRITE(IOUT,*) 'Upper (2nd) Shield material = CONCRETE'
        ELSEIF(MAT(2).EQ.3) THEN
          WRITE(IOUT,*) 'Upper (2nd) Shield material = IRON'
        ELSE
          WRITE(IOUT,*) 'Upper (2nd) Shield material = LEAD'
        END IF

        IF (T(2).LT.0.0) THEN
          WRITE(IOUT,957) -T(2)
        ELSE
          WRITE(IOUT,956) T(2)
        END IF
        WRITE(IOUT,*)' '

 958    IF(NTYPE.EQ.1) THEN
          WRITE(IOUT,*) 'Source geometry =  SILO COLLIMATION'
          WRITE(IOUT,959) ACOLL
        ELSE
          WRITE(IOUT,*) 'Source geometry = CONVEX POLYGON COLLIMATION'
          WRITE(IOUT,961) NEDGE
          DO 963 IVERT=1,NEDGE
            WRITE(IOUT,962) IVERT,XEDGE(IVERT),IVERT,YEDGE(IVERT)
 963      CONTINUE
        END IF

C****** PAUSE SCREEN DISPLAY
       IF (IOUT.EQ.6) THEN
         WRITE(IOUT,*) ' '
         WRITE(IOUT,*) ' '
         PAUSE '---> Press ENTER to see skyshine doses'
       ENDIF

C****** CALCULATE AN ERROR ESTIMATE USING THE VALUES FROM EACH
C       BATCH

        WRITE (IOUT,951)
        DO 954 I=1,NDIST+1
          XSD = XPHI(I)
          RAVG = SUM1(I)/NBATCH
          POPVAR = (SUM2(I)-SUM1(I)*SUM1(I)/NBATCH)/(NBATCH-1)
          IF(POPVAR.LE.0.0D0) THEN
              AVGVAR=0.0
          ELSE
              AVGVAR = DSQRT(POPVAR/NBATCH)
          ENDIF

C****** OUTPUT DOSE RESULTS
          WRITE (IOUT,952) XSD,XSD*RHOACT*100.D0,RAVG,
     *                     RAVG*1.145D0,AVGVAR*100.D0/RAVG

 954    CONTINUE
        WRITE(IOUT,960)

 951    FORMAT(//24X,' SKYSHINE DOSES ',/1X,63('-'),/
     1  6X,'X(m)',6X,'g/cm^2',6X,'rad/photon',4X,'R/photon',8X,'+-%',/
     1  1X,63('-'))
 960    FORMAT(1X,63('-')/)
 952    FORMAT (F10.2,2X,F10.2,2X,1PE13.2,1PE13.2,2X,0PF10.2)

 955    FORMAT (///20('*'),' MCSKY RESULTS ',20('*'),/
     *    ' Number of histories                      = ',I8,/
     *    ' Source photon energy (MeV)               = ',F8.3,/
     *    ' Lower shield elevation above source (m)  = ',F8.2,/
     *    ' Detector elevation above source (m)      = ',F8.2,/
     *    ' Maximum source to detector distance (m)  = ',F8.2,/)
 956    FORMAT( ' Shield thickness (cm) = ',F8.2,/)
 957    FORMAT( ' Shield thickness (mfp) = ',F8.2,/)
 959    FORMAT( ' Collimation conical half-angle (degrees)  = ',F8.2)
 961    FORMAT(1X,I2,' vertices: x-y coordinates are:')
 962    FORMAT(5X,'X(',I2,') = ',F8.2,'    Y(',I2,') = ',F8.2)

        RETURN
      END




      BLOCK DATA EAGRID
C**********************************************************************
C  Initializ data for XBEAM
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /GAMVEC/ EE(2,15),PPSI(2,20)
C--- Dicrete angles for calculation of approximate lbrf
C      Energy < 15.0 MeV
        DATA (PPSI(1,I),I=1,20)
     *       /0.5,1.5,2.5,4.0,6.0,8.5,12.5,17.5,25.0,35.0,45.0,
     *        55.0,65.0,75.0,85.0,95.0,110.,130.,150.,170./
C      Energy > 10.0 MeV
        DATA (PPSI(2,I),I=1,17)
     *       /2.0,3.5,5.0,7.5,10.0,15.0,20.0,30.0,45.0,60.0,
     *        75.0,85.0,95.0,105.0,120.0,135.0,150.0/

C--- Discrete energies for calculation of approximate lbrf
C      Energy < 10.0 MeV
        DATA (EE(1,I),I=1,15)
     *       /0.02,0.03,0.04,0.06,0.08,0.1,0.2,0.4,0.7,1.0,
     *        2.0,4.0,7.0,10.,15./
C      Energy > 10.0 MeV
        DATA (EE(2,I),I=1,5)/10.0,20.0,40.0,70.0,100.0/
      END



      SUBROUTINE XBINI(IPARM)
C***********************************************************************
C  Purpose:
C    Reads data files containing coefficients for the three-parameter
C    approximation to the LBRF.  Call once before using subprogram
C    XBEAM.
C***********************************************************************
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        CHARACTER*15 EHDR
        COMMON /GAMDAT/ G(2,15,20,3),ANGL(2,20)
        COMMON /GAMDT4/ G4(15,20,4),ANG4(20)

C****** Read in the fit parameters for the approximate LBRF
        OPEN(21,FILE='LOWGAM.DAT')
        OPEN(23,FILE='HIGHGAM.DAT')

C** Read low-E 3-parm data
        DO 20 I=1,15
         READ(21,10) EHDR
         READ(21,*) (ANGL(1,J),G(1,I,J,1),G(1,I,J,2),G(1,I,J,3),J=1,20)
 20     CONTINUE

C** Read high-E 3-parm data
        DO 30 I=1,5
         READ(23,10) EHDR
         READ(23,*) (ANGL(2,J),G(2,I,J,1),G(2,I,J,2),G(2,I,J,3),J=1,17)
 30     CONTINUE

C** Read low-E 4-parm data if needed
        IF(IPARM.EQ.4) THEN
          OPEN(24,FILE='LOWGAM4.DAT')
          DO 40 I=1,15
           READ(24,10) EHDR
           READ(24,*) (ANG4(J),G4(I,J,1),G4(I,J,2),G4(I,J,3),
     *                                             G4(I,J,4),J=1,20)
 40       CONTINUE
          CLOSE(24)
        END IF

 10     FORMAT(A)
        CLOSE(21)
        CLOSE(23)
      END



      SUBROUTINE XBEAM4(E,PSI,ZZ,RHOACT,RESP)
C**********************************************************************
C  Purpose:
C    This subroutine calculates the skyshine dose at a detector
C    arising from a monoenergetic and monodirectional photon beam
c    using a 3-parameter approximation for the LBRF.
C
C  Input:
C      E = Source photon energy (MeV),
C    PSI = photon emission angle (deg.)
C     ZZ = source-to-detector distance.
C
C  Output:
C     RESP = Line-beam response function (rad/photon)
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent enrgy index
C        JP     = angular group index in which a beam angle reside
C        JPADJ  = adjacent angle index
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        COMMON /GAMDT4/ G4(15,20,4),ANG4(20)
        COMMON /GAMVEC/ EE(2,15),PPSI(2,20)

        NE = 15
        NANG = 20
        FK=1.308E-11
        RHOAIR = 0.0012D0
        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (E.LE.EE(1,1)) THEN
            IE = 1
        ELSEIF (E.GE.EE(1,NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(E.GE.EE(1,I-1).AND.E.LT.EE(1,I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EE(1,IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.PPSI(1,1)) THEN
            JP = 1
        ELSEIF (PSI.GE.PPSI(1,NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.PPSI(1,I-1).AND.PSI.LT.PPSI(1,I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate LBRF at four surrounding tabulated grid points
        XX=ZZ*RFACT
        FIE(JP)=DEXP(G4(IE,JP,1) - G4(IE,JP,3)*XX)
     *          *XX**(G4(IE,JP,2) - XX*G4(IE,JP,4))
        FIEADJ(JP)=DEXP(G4(IEADJ,JP,1)-G4(IEADJ,JP,3)*XX)
     *             *XX**(G4(IEADJ,JP,2)-XX*G4(IEADJ,JP,4))
        FIE(JPADJ)=DEXP(G4(IE,JPADJ,1)-G4(IE,JPADJ,3)*XX)
     *             *XX**(G4(IE,JPADJ,2)-XX*G4(IE,JPADJ,4))
        FIEADJ(JPADJ)=DEXP(G4(IEADJ,JPADJ,1)
     *             - G4(IEADJ,JPADJ,3)*XX)
     *             *XX**(G4(IEADJ,JPADJ,2)-XX*G4(IEADJ,JPADJ,4))

C** Interpolate between the bracketing energies
        EFACT=(E-EE(1,IE))/(EE(1,IEADJ)-EE(1,IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT

C** Interpolate between the bracketing angles

        FF(JP) = DLOG(FF(JP))
        FF(JPADJ) = DLOG(FF(JPADJ))
        RESP=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-PPSI(1,JP))/
     *              (PPSI(1,JPADJ)-PPSI(1,JP))

        RESP=E*FK*(RFACT*RFACT)*DEXP(RESP)
        RETURN
      END



      SUBROUTINE XBEAM(E,PSI,ZZ,RHOACT,RESP,IPARM)
C**********************************************************************
C  Purpose:
C    This subroutine calculates the skyshine dose at a detector
C    arising from a monoenergetic and monodirectional photon beam
c    using a 3-parameter approximation for the LBRF.
C
C  Input:
C      E = Source photon energy (MeV),
C    PSI = photon emission angle (deg.)
C     ZZ = source-to-detector distance.
C
C  Output:
C     RESP = Line-beam response function (rad/photon)
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent enrgy index
C        JP     = angular group index in which a beam angle reside
C        JPADJ  = adjacent angle index
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        COMMON /GAMDAT/ G(2,15,20,3),ANGL(2,20)
        COMMON /GAMVEC/ EE(2,15),PPSI(2,20)

        IF(E.LT.15.0) THEN
          IF (IPARM.EQ.4) THEN
            CALL XBEAM4(E,PSI,ZZ,RHOACT,RESP)
            RETURN
          END IF
          IDAT = 1
          NE = 15
          NANG = 20
        ELSE
          IDAT = 2
          NE = 5
          NANG = 17
        END IF

        FK=1.308E-11
        RHOAIR = 0.0012D0
        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (E.LE.EE(IDAT,1)) THEN
            IE = 1
        ELSEIF (E.GE.EE(IDAT,NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(E.GE.EE(IDAT,I-1).AND.E.LT.EE(IDAT,I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EE(IDAT,IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.PPSI(IDAT,1)) THEN
            JP = 1
        ELSEIF (PSI.GE.PPSI(IDAT,NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.PPSI(IDAT,I-1).AND.PSI.LT.PPSI(IDAT,I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate LBRF at four surrounding tabulated grid points
        XX=ZZ*RFACT
        FIE(JP)=DEXP(G(IDAT,IE,JP,1) - G(IDAT,IE,JP,3)*XX)
     *          *XX**G(IDAT,IE,JP,2)
        FIEADJ(JP)=DEXP(G(IDAT,IEADJ,JP,1)-G(IDAT,IEADJ,JP,3)*XX)
     *             *XX**G(IDAT,IEADJ,JP,2)
        FIE(JPADJ)=DEXP(G(IDAT,IE,JPADJ,1)-G(IDAT,IE,JPADJ,3)*XX)
     *             *XX**G(IDAT,IE,JPADJ,2)
        FIEADJ(JPADJ)=DEXP(G(IDAT,IEADJ,JPADJ,1)
     *             - G(IDAT,IEADJ,JPADJ,3)*XX)
     *             *XX**G(IDAT,IEADJ,JPADJ,2)

C** Interpolate between bracketing energies
        EFACT=(E-EE(IDAT,IE))/(EE(IDAT,IEADJ)-EE(IDAT,IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT

C** Interpolate between the bracketing angles

        FF(JP) = DLOG(FF(JP))
        FF(JPADJ) = DLOG(FF(JPADJ))
        RESP=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-PPSI(IDAT,JP))/
     *              (PPSI(IDAT,JPADJ)-PPSI(IDAT,JP))

        RESP=E*FK*(RFACT*RFACT)*DEXP(RESP)
        RETURN
      END


      BLOCK DATA EACRSS
C**********************************************************************
C Initialize nuclear data for common arrays
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /BINS/ NBE,NBA,NBATCH
        COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)  !<<<<<<<<< Revised Oct 97, REF
        COMMON /DENSTY/ DENS(4)
        COMMON /LIMIT/ ELIM(51),PHILIM(21)

        DATA NBE/50/
        DATA NBA/20/
        DATA NBATCH/10/

C***** Material Atomic Masses
        DATA AMASS/26.98D0,18.77D0,55.85D0,207.2D0/

C***** Material Densities
        DATA DENS/2.70D0,2.3D0,7.87D0,11.36D0/

C***** Aluminum Cross Section coefficients
C      Compton
        DATA (AMU(1,1,I),I=1,11)
     *         / 4.391341796557888D-1,-4.909051805986715D-1,
     *          -1.333330805413437D-1, 1.614059419558586D-3,
     *           6.789829651130458D-3, 4.317490898684908D-3,
     *          -5.974345141883735D-3, 7.388924181587273D-4,
     *           1.262902599794703D-3,-1.658501408178103D-4,
     *          -1.006822610852379D-4/
C      Pair production
        DATA (AMU(1,2,I),I=1,11)
     *         /-6.373960271473098D-1, 4.144453441893409D-1,
     *          -3.059956148018670D-1, 1.713831973595873D-1,
     *           2.412588538904213D-3,-9.196943578858210D-2,
     *           0.000000000000000D00, 0.000000000000000D00,
     *           0.000000000000000D00, 0.000000000000000D00,
     *           0.000000000000000D00/
C      Total
        DATA (AMU(1,3,I),I=1,11)
     *         /-1.200619658972617D00,-4.568092020653343D-1,
     *          -1.996299151204586D-1, 5.227054165344491D-2,
     *           2.345528642780792D-1,-3.706212872577459D-2,
     *          -3.519049958242595D-2,-4.271922973154483D-3,
     *           5.401614095486342D-5, 1.442571806425298D-3,
     *           3.775598476858302D-4/

C***** Concrete Cross Section coefficients
C      Compton
        DATA (AMU(2,1,I),I=1,11)
     *         / 2.969873063723992D-1,-4.898962041605531D-1,
     *          -1.229013427475533D-1,-0.646042438194109D-2,
     *          -8.955387331405081D-3, 1.273846029182773D-2,
     *           3.066625843844734D-3,-2.537391622802159D-3,
     *          -7.615960331765083D-4, 2.111273503725030D-4,
     *           6.603048728152942D-5/
C      Pair Production
        DATA (AMU(2,2,I),I=1,11)
     *         /-6.835996990535956D-1, 3.831788519374033D-2,
     *          -3.230724528737545D-1, 5.016074153247182D-1,
     *           3.145567531737921D-3,-1.821769281741999D-1,
     *          -5.359962004775433D-6, 0.000000000000000D00,
     *           0.000000000000000D00, 0.000000000000000D00,
     *           0.000000000000000D00/
C      Total
        DATA (AMU(2,3,I),I=1,11)
     *         /-1.184627440996852D00,-5.146797263650505D-1,
     *          -2.071208138484681D-1, 2.263952251850396D-1,
     *           2.396368130364689D-1,-1.913777689044947D-1,
     *          -4.736324304886239D-2, 4.417664480210789D-2,
     *           6.998363566465241D-3,-3.580232225429919D-3,
     *          -6.775714955896409D-4/

C***** Iron Cross Section coefficients
C      Compton
        DATA (AMU(3,1,I),I=1,11)
     *         / 7.392914301170435D-1,-4.912468675270767D-1,
     *          -1.323614085624653D-1, 1.080656001767565D-2,
     *          -3.433418845498458D-3, 2.251882579071866D-3,
     *          -2.088473330555031D-3, 1.088744880437814D-3,
     *           6.479631821745760D-4,-1.931685705320679D-4,
     *          -7.014810420472226D-5/
C      Pair Production
        DATA (AMU(3,2,I),I=1,11)
     *         /-1.038225251032108D-1, 2.640627359151460D-1,
     *          -2.396625285077042D-1, 3.785819878275352D-1,
     *          -5.779658784808552D-3,-1.592602335333242D-1,
     *           7.105008468865731D-5, 0.000000000000000D00,
     *           0.000000000000000D00, 0.000000000000000D00,
     *           0.000000000000000D00/
C      Total
        DATA (AMU(3,3,I),I=1,11)
     *         /-1.206693022641418D00,-4.676947983050160D-1,
     *          -2.251650587667903D-1, 8.288550791176554D-2,
     *           6.719545611098675D-1,-2.347963979106667D-1,
     *          -2.576044684816875D-1, 7.768560176719180D-2,
     *           4.421444772274438D-2,-7.473524643273419D-3,
     *          -3.099864841804840D-3/

C***** Lead Cross Section coefficients
C      Compton
        DATA (AMU(4,1,I),I=1,11)
     *         / 1.234872834596074D00,-4.797065925799817D-1,
     *          -1.402754908211176D-1, 1.072222316527636D-2,
     *          -1.826039273689403D-2, 1.573143743617556D-2,
     *           3.230544088821196D-3,-3.535752991367582D-3,
     *          -4.862182919824038D-4, 2.769450005433402D-4,
     *           4.784213469525582D-5/
C      Pair Production
        DATA (AMU(4,2,I),I=1,11)
     *         / 4.072170292542221D00,-4.430731013307036D00,
     *          -1.339052249017441D00, 4.010637290519840D00,
     *           1.812588223624467D-1,-1.649172061621550D00,
     *          -1.039367294661333D-2, 2.539800975368867D-1,
     *           0.000000000000000D00, 0.000000000000000D00,
     *           0.000000000000000D00/
C      Total
        DATA (AMU(4,3,I),I=1,11)
     *         /-1.147165006083587D00,-9.814462739047836D-1,
     *           9.623223973996474D-1, 1.515531519018430D-1,
     *          -3.101463799281304D-1,-5.861809564272871D-2,
     *           7.655510638722211D-2, 1.015124052477078D-2,
     *          -9.065620995779955D-3,-7.197655928868791D-4,
     *           3.385099486318057D-4/

C***** Energy and Angle Bin Limits
        DATA (ELIM(I),I=1,51)  /0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0,0.000D0,0.000D0,0.000D0,0.000D0,
     *                          0.000D0/

       DATA (PHILIM(I),I=1,21)/0.000D0,1.000D0,2.000D0,3.000D0,5.000D0,
     *                         7.000D0,10.00D0,15.00D0,20.00D0,30.00D0,
     *                         40.00D0,50.00D0,60.00D0,70.00D0,80.00D0,
     *                         90.00D0,100.0D0,120.0D0,140.0D0,160.0D0,
     *                         180.0D0/

      END
*

      SUBROUTINE SHIELD(NDIST,NBE,NBA,MA,EIN,Z1,W1,P1)
C**********************************************************************
C  Purpose:
C    This subroutine performs a Monte Carlo "walk" of the photon
C    through the shield.
C
C  Input variables:
C    NDIST = number of detector locations
C    NBE   = number of energy bins to be sued
C    NBA   = number of angle bins to be sued
C    MA    = material numbr for shield
C    EIN = photon energy (MeV)
C    Z1  = incident z coordinate of photon (m) (bottom of shield)
C    W1  = incident cosine of polar angle of photon direction        !<<<<<<<<< Revised Oct 97, REF
C    P1  = incident azimuthal  angle of photon direction (radians)   !<<<<<<<<< Revised Oct 97, REF
C
C  Output variables (in COMMON blocks):
C    IEPHI  = collided escaping photon energy-angle distribution
C    IPPPHI = uncollided escaping pair production angle distribution
C    ISPHI  = uncollided source photon angle distribution
C
C  Other variables used:
C    NPAIR = flag used to determine if a pair production photon has
C            yet to be tracked (0 - no, 1 - yes)
C    NUNC  = flag used to determine what type of photon emitted
C            (0 - source, 1 - pair production (0.511 MeV) ,
C             2 - collided (i.e. not source or pair production))
C
C  NEEDED SUBROUTINES:
C    HOWFAR = determines where the photon is in the shield
C    DIRCOS = determines the direction cosines of photon direction
C    BIN    = collided photon energy-angle distribution
C    BINPP  = uncollided pair production photon angle distribution
C    BINMN  = uncollided source photon angle distribution
C
C  NEEDED FUNCTIONS:
C    FLTRN = random number generator
C    PATHL = calculates a path length for the photon (cm)
C    CMPT  = calculates the compton scattering cross section (cm^2/g)
C    PPAIR = calculates the pair production cross section (cm^2/g)
C    TOT   = calculates the total scattering cross section (cm^2/g)
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /DISTMA/ MAT(2)
        COMMON /PHIX/ XPHI(20)
        COMMON /BINEP/ IEPHI(20,50,20)
        COMMON /PPBIN/ IPPPHI(20,20)
        COMMON /MNBIN/ ISPHI(20,20)
        COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)      !<<<<<<<<< Revised Oct 97, REF
        COMMON /DENSTY/ DENS(4)
        COMMON /LIMIT/ ELIM(51),PHILIM(21)
        COMMON /PARKD/ SEED

C---- Initialize flags for pair production and photon scoring type

        NPAIR = 0
        NUNC = 0
        MTEMP = MA                                         !<<<<<<<<< Revised Oct 97, REF

C---- Calculate total cross section

 450    AENG = DLOG10(EIN)

        DO K=1,4                                           !<<<<<<<<< Revised Oct 97, REF
          XMUTOT(K) = TOT(AENG,K)                          !<<<<<<<<< Revised Oct 97, REF
        END DO

C---- Determine path length

        IF((MAT(1).EQ.0).AND.(MAT(2).EQ.0))THEN            !<<<<<<<<< Revised Oct 97, REF
            GO TO 580                                      !<<<<<<<<< Revised Oct 97, REF
        ELSE                                               !<<<<<<<<< Revised Oct 97, REF
            S = PATHL(XMUTOT(MTEMP),MTEMP)                 !<<<<<<<<< Revised Oct 97, REF
        END IF                                             !<<<<<<<<< Revised Oct 97, REF

C---- Determine if photon is reflected, in shield, or transmitted

        CALL HOWFAR(MTEMP,Z1,W1,S,Z2,IDISC)
        Z1 = Z2

 500    IF (IDISC.EQ.0) THEN

C---- End subroutine due to reflection

          GOTO 590
        ELSEIF (IDISC.EQ.99) THEN

C---- Skip to binning/scoring section

          GOTO 580
        END IF

C---- Set photon type flag to 2

        NUNC = 2

C---- Calculate photon Compton wavelength

        XLAM1 = (0.511D0)/EIN

C---- Calculate Compton and pair productin cross sections

        XMUCOM = COM(AENG,MTEMP)
        XMUPP = PPAIR(AENG,MTEMP)
        RATIO1 = XMUCOM/XMUTOT(MTEMP)                  !<<<<<<<<< Revised Oct 97, REF
        RATIO2 = (XMUCOM+XMUPP)/XMUTOT(MTEMP)          !<<<<<<<<< Revised Oct 97, REF
 550    TEST1 = FLTRN()

        IF (TEST1.GT.RATIO2) THEN

C****** Photon is absorbed

          GOTO 590
        ELSEIF (TEST1.GT.RATIO1) THEN

C***** Photon absorbed two pair production photons generated
C----  Set flag for pair production
          NPAIR = 1
C----  Get new cosine of polar angle and azimuthal angle
          CALL GETWP(W1,P1)
C----  Save necessary data for second photon
          ZPP = Z1
          XPI = 3.14159265358979D0
          WPP = DCOS(XPI - DACOS(W1))
          IF ((P1+XPI).GT.(2.0D0*XPI)) THEN
            PPP = P1 - XPI
          ELSE
            PPP = P1 + XPI
          END IF
          EIN = 0.511D0
C---- Set flag to photon type to pair production
          NUNC = 1
C---- Restart tracking at new location and energy
          GOTO 450
        END IF

C**** If photon is compton scattered process is continued
C---- Determine new wavelength after scatter
        CALL CMPT(XLAM1,XLAM2)

C---- Calculate cosine of scattering angle
        COSS = 1.0D0 + XLAM1 - XLAM2

C---- Calculate energy of scattered photon
        EOUT = EIN/(1.0D0+(1.0D0-COSS)/XLAM1)

C---- If energy is below cutoff energy then end tracking
        IF (EOUT.LT.0.02D0) GOTO 590

C---- Calculate new azimuthal angle and cosine of polar angle
        CALL SHIFT(W1,P1,COSS,W2,P2)
        P1 = P2
        EIN = EOUT
        W1 = W2

C---- Repeat path length-collision process
        GOTO 450

C*********** Binning/scoring section
C---- Bin the photon in energy and/or angle for each distance
 580    DO 581 M=1,NDIST+1
C---- Calculate the cosine of the emission angle with
C     respect to the source-to-detector axis
          ZDUM = (HDET-((T1+T2)/100.D0)-HSLAB)/XPHI(M)
          ZETA = DATAN(ZDUM)
          WPHI = DSQRT(1-W1*W1)*DCOS(P1)*DCOS(ZETA) + W1*DSIN(ZETA)

          IF (NUNC.EQ.2) THEN
C---- Photon is an escaping collided photon
            CALL BIN(NBE,NBA,M,WPHI,EIN)
          ELSEIF (NUNC.EQ.1) THEN
C---- Photon is an escaping pair production photon
            CALL BINPP(NBA,M,WPHI)
          ELSE
C---- Photon is an escaping source photon
            CALL BINMN(NBA,M,WPHI)
          END IF
 581    CONTINUE

C******* If a pair production photon is waiting to be tracked
C        reset necessary variables to stored values and track
 590    IF (NPAIR.GT.0) THEN
          Z1 = ZPP
          W1 = WPP
          P1 = PPP
          EIN = 0.511D0
          NPAIR = 0
          NUNC = 1
          GOTO 450
        END IF

        RETURN
      END

      SUBROUTINE HOWFAR (MTEMP,Z1,W1,S,Z2,IDISC)
C**********************************************************************
C  Purpose:
C    Determines if the photon has escaped or is in the shield and if
C    it is in the shield its position in the shield.
C
C  Input:
C    Z1 = z axis position of photon (cm)
C    W1 = direction cosine of photon
C    S  = path length of photon (cm)
C
C  Output:
C    Z2 = new z axis position of photon (cm)
C**********************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /DISTNC/ HDET,HSLAB,HSL,XSTOD,T(2),T1,T2
        COMMON /CROSS/ AMU(4,3,11),AMASS(4),XMUTOT(4)      !<<<<<<<<< Revised Oct 97, REF
        COMMON /DISTMA/ MAT(2)
        COMMON /DENSTY/ DENS(4)

C--- Calculate new z-axis position
        Z2 = Z1 + S*W1

*-----------------------------------------------------------------------

C---  Revised Oct 97, REF

        IF    ((Z1.GE.(HSL+T1)).AND.(Z2.GE.(HSL+T1))) THEN   ! both points in upper zone
           IDISC = 1
           MTEMP=MAT(2)
           IF(Z2.GE.(HSL+T1+T2)) THEN                        ! transmission
              IDISC = 99
           END IF
        ELSEIF((Z1.LE.(HSL+T1)).AND.(Z2.LE.(HSL+T1))) THEN   ! both points in lower zone
           IDISC = 1
           MTEMP=MAT(1)
           IF(Z2.LE.(HSL      )) THEN                        ! reflection
              IDISC = 0
           END IF
        ELSEIF((Z1.LE.(HSL+T1)).AND.(Z2.GT.(HSL+T1))) THEN   ! lower zone to upper
           IF(MAT(2).EQ.0) THEN                              ! transmission
              IDISC = 99
           ELSE
             Z2=HSL+T1 -(HSL+T1-Z2)                          ! correct second leg
     &        *XMUTOT(MAT(1))*DENS(MAT(1))/XMUTOT(MAT(2))/DENS(MAT(2))
             MTEMP = MAT(2)                                  ! revert material no.
             IDISC = 1                                       ! keep tracking
             IF(Z2.GE.(HSL+T1+T2)) THEN                      ! transmission
               IDISC = 99
             END IF
           END IF
        ELSEIF((Z1.GT.(HSL+T1)).AND.(Z2.LE.(HSL+T1))) THEN   ! upper zone to lower
           IF(MAT(1).EQ.0) THEN                              ! reflection
              IDISC = 0
           ELSE
              Z2=HSL+T1 -(HSL+T1-Z2)                         ! correct second leg
     &         *XMUTOT(MAT(2))*DENS(MAT(2))/XMUTOT(MAT(1))/DENS(MAT(1))
              MTEMP = MAT(1)                                 ! revert material no.
              IDISC = 1                                      ! keep tracking
              IF(Z2.LE.(HSL      )) THEN                     ! reflection
                 IDISC = 0
              END IF
           END IF
        END IF
        RETURN
      END
